#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          02-07-2018
//      Created for Project :   FLAME
"""

from sqlalchemy import Column, String, Integer, UniqueConstraint, and_
from clmcservice.models.meta import Base


class ServiceFunctionEndpoint(Base):
    """
    This class defines the main model of the WHOAMI API, declaring the global tags for a specific service function on a specific endpoint.
    """

    __tablename__ = 'sfendpoint'  # table name in the PostgreSQL database

    __table_args__ = (UniqueConstraint('sf', 'sf_endpoint'),)  # defines a unique constraint across 2 columns - sf, sf_endpoint

    server = Column(String, nullable=False)  # cluster label
    location = Column(String, nullable=False)  # location label
    sfc = Column(String, nullable=False)  # service function chain label
    sfc_instance = Column(String, nullable=False)  # service function chain instance identifier
    sf_package = Column(String, nullable=False)  # service function package label
    sf = Column(String, nullable=False)   # service function node defined in the TOSCA resource specification
    sf_endpoint = Column(String, nullable=False, primary_key=True)  # service function endpoint (FQDN(s) + IP address)

    @property
    def json(self):
        """
        Converts an instance of a ServiceFunctionEndpoint to JSON format.

        :return: a python dictionary object
        """

        fields = {c.name: getattr(self, c.name) for c in self.__table__.columns}

        return fields

    @staticmethod
    def get(sf_endpoint):
        """
        Gets the service function endpoint object or None if not existing.

        :param sf_endpoint: service function endpoint

        :return: the first object from the result set that matches the unique constraint (should be only one) or None
        """

        return ServiceFunctionEndpoint.query().filter(ServiceFunctionEndpoint.sf_endpoint == sf_endpoint).first()

    @staticmethod
    def exists(sf_endpoint):
        """
        Checks if an instance matching the sf_endpoint ID exists.

        :param sf_endpoint: service function endpoint ID to check

        :return: True if exists, False otherwise
        """

        return ServiceFunctionEndpoint.get(sf_endpoint) is not None
